@@ -8,7 +8,6 @@ fi
 
 set -e -x -v
 
-# The docker image this is running in has NSS sources.
 # Get the HACL* source, containing a snapshot of the C code, extracted on the
 # HACL CI.
 git clone -q "https://github.com/hacl-star/hacl-star" ~/hacl-star
@@ -16,11 +15,20 @@ git -C ~/hacl-star checkout -q 0f136f28935822579c244f287e1d2a1908a7e552
 
 # Format the C snapshot.
 cd ~/hacl-star/dist/mozilla
-cp ~/nss/.clang-format .
+cp ${VCS_PATH}/nss/.clang-format .
 find . -type f -name '*.[ch]' -exec clang-format -i {} \+
 cd ~/hacl-star/dist/karamel
-cp ~/nss/.clang-format .
+cp ${VCS_PATH}/nss/.clang-format .
 find . -type f -name '*.[ch]' -exec clang-format -i {} \+
+cd ~/hacl-star/dist/gcc-compatible
+cp ${VCS_PATH}/nss/.clang-format .
+find . -type f -name '*.[ch]' -exec clang-format -i {} \+
+
+cd ~/hacl-star
+patches=(${VCS_PATH}/nss/automation/taskcluster/scripts/patches/*.patch)
+for f in "${patches[@]}"; do
+    git apply "$f"
+done
 
 # These diff commands will return 1 if there are differences and stop the script.
 
@@ -30,22 +38,19 @@ find . -type f -name '*.[ch]' -exec clang-format -i {} \+
 # For instance, the files Hacl_Chacha20.h are present in both directories, but the content differs.
 
 # TODO(Bug 1899443): remove these exceptions
-files=($(find ~/nss/lib/freebl/verified/internal -type f -name '*.[ch]'))
+files=($(find ${VCS_PATH}/nss/lib/freebl/verified/internal -type f -name '*.[ch]' -not -path "*/freebl/verified/internal/libcrux*"))
 for f in "${files[@]}"; do
     file_name=$(basename "$f")
     hacl_file=($(find ~/hacl-star/dist/mozilla/internal/ -type f -name $file_name))
     if [ $file_name == "Hacl_Ed25519.h" \
-        -o $file_name == "Hacl_Ed25519_PrecompTable.h" \
-        -o $file_name == "libcrux_sha3_internal.h" \
-        -o $file_name == "libcrux_core.h" \
-        -o $file_name == "libcrux_mlkem_portable.h" ]
+        -o $file_name == "Hacl_Ed25519_PrecompTable.h" ]
     then
-        continue;
+        continue
     fi
-    diff $hacl_file $f
+    diff -u $hacl_file $f
 done
 
-files=($(find ~/nss/lib/freebl/verified/ -type f -name '*.[ch]' -not -path "*/freebl/verified/internal/*" -not -path "*/freebl/verified/config.h"))
+files=($(find ${VCS_PATH}/nss/lib/freebl/verified/ -type f -name '*.[ch]' -not -path "*/freebl/verified/internal/*" -not -path "*/freebl/verified/config.h" -not -path "*/freebl/verified/libcrux*"))
 for f in "${files[@]}"; do
     file_name=$(basename "$f")
     hacl_file=($(find ~/hacl-star/dist/mozilla/ ~/hacl-star/dist/karamel/ -type f -name $file_name -not -path "*/hacl-star/dist/mozilla/internal/*"))
@@ -53,67 +58,43 @@ for f in "${files[@]}"; do
         -o $file_name == "Hacl_P384.h" \
         -o $file_name == "Hacl_P521.c" \
         -o $file_name == "Hacl_P521.h" \
-        -o $file_name == "libcrux_mlkem_portable.c" \
-        -o $file_name == "libcrux_sha3_internal.h" \
-        -o $file_name == "libcrux_core.h" \
+        -o $file_name == "eurydice_glue.h" \
         -o $file_name == "target.h" ]
     then
-        continue;
+        continue
     fi
 
     if [ $file_name == "Hacl_Ed25519.h"  \
         -o $file_name == "Hacl_Ed25519.c" ]
     then
-        continue;
+        continue
     fi
-    diff $hacl_file $f
+    diff -u $hacl_file $f
 done
 
 # Here we process the code that's not located in /hacl-star/dist/mozilla/ but
 # /hacl-star/dist/gcc-compatible. 
 
-cd ~/hacl-star/dist/gcc-compatible
-cp ~/nss/.clang-format .
-find . -type f -name '*.[ch]' -exec clang-format -i {} \+
-
-patches=($(find ~/nss/automation/taskcluster/scripts/patches/ -type f -name '*.patch'))
-for f in "${patches[@]}"; do
-    file_name=$(basename "$f")
-    file_name="${file_name%.*}"
-    if_internal="${file_name##*.}"
-    if [ $if_internal == "internal" ]
-    then
-        file_name="${file_name%.*}"
-        patch_file=($(find ~/hacl-star/dist/gcc-compatible/internal/ -type f -name $file_name))
-    else
-        patch_file=($(find ~/hacl-star/dist/gcc-compatible/ -type f -name $file_name -not -path "*/hacl-star/dist/gcc-compatible/internal/*"))
-    fi
-    if [ ! -z "$patch_file" ]
-    then
-        patch $patch_file $f
-    fi
-done
-
-files=($(find ~/nss/lib/freebl/verified/internal -type f -name '*.[ch]'))
+files=($(find ${VCS_PATH}/nss/lib/freebl/verified/internal -type f -name '*.[ch]'))
 for f in "${files[@]}"; do
     file_name=$(basename "$f")
     hacl_file=($(find ~/hacl-star/dist/gcc-compatible/internal/ -type f -name $file_name))
     if [ $file_name != "Hacl_Ed25519.h" \
         -a $file_name != "Hacl_Ed25519_PrecompTable.h" ]
     then
-        continue;
+        continue
     fi  
-    diff $hacl_file $f
+    diff -u $hacl_file $f
 done
 
-files=($(find ~/nss/lib/freebl/verified/ -type f -name '*.[ch]' -not -path "*/freebl/verified/internal/*"))
+files=($(find ${VCS_PATH}/nss/lib/freebl/verified/ -type f -name '*.[ch]' -not -path "*/freebl/verified/internal/*"))
 for f in "${files[@]}"; do
     file_name=$(basename "$f")
     hacl_file=($(find ~/hacl-star/dist/gcc-compatible/ -type f -name $file_name -not -path "*/hacl-star/dist/gcc-compatible/internal/*"))
     if [ $file_name != "Hacl_Ed25519.h" \
         -a $file_name != "Hacl_Ed25519.c" ]
     then
-        continue;
+        continue
     fi  
-    diff $hacl_file $f
+    diff -u $hacl_file $f
 done
