@@ -5,13 +5,14 @@ import sys
 
 def main():
     if len(sys.argv) < 2:
-        raise Exception('Specify either "asan", "fuzzer", "msan", "sancov", "sourcecov" or "ubsan" as argument.')
+        raise Exception('Specify either "asan", "fuzzer", "msan", "sancov", "sourcecov", "tsan" or "ubsan" as argument.')
 
     sanitizer = sys.argv[1]
     if sanitizer == "ubsan":
         if len(sys.argv) < 3:
             raise Exception('ubsan requires another argument.')
-        print('-fsanitize='+sys.argv[2]+' -fno-sanitize-recover=undefined ', end='')
+        print('-fsanitize='+sys.argv[2]+' -fno-sanitize-recover='+sys.argv[2] + ' ', end='')
+        print('-fno-sanitize=nonnull-attribute -fno-sanitize=enum ', end='')
         return
     if sanitizer == "asan":
         print('-fsanitize=address -fsanitize-address-use-after-scope ', end='')
@@ -29,6 +30,10 @@ def main():
     if sanitizer == "sourcecov":
         print('-fprofile-instr-generate -fcoverage-mapping', end='')
         return
+    if sanitizer == "tsan":
+        print('-fsanitize=thread ', end='')
+        print('-fno-omit-frame-pointer -fno-optimize-sibling-calls ', end='')
+        return
     if sanitizer == "fuzzer":
         print('-fsanitize=fuzzer-no-link ', end='')
         return
