@@ -602,7 +602,7 @@ SGN_Digest(SECKEYPrivateKey *privKey,
     }
     /* check the policy on the encryption algorithm */
     enctag = sec_GetEncAlgFromSigAlg(
-        SEC_GetSignatureAlgorithmOidTag(privKey->keyType, algtag));
+        SEC_GetSignatureAlgorithmOidTagByKey(privKey, NULL, algtag));
     if ((enctag == SEC_OID_UNKNOWN) ||
         (NSS_GetAlgorithmPolicy(enctag, &policyFlags) == SECFailure) ||
         !(policyFlags & NSS_USE_ALG_IN_ANY_SIGNATURE)) {
@@ -675,6 +675,9 @@ SEC_GetSignatureAlgorithmOidTag(KeyType keyType, SECOidTag hashAlgTag)
     SECOidTag sigTag = SEC_OID_UNKNOWN;
 
     switch (keyType) {
+        case rsaPssKey:
+            sigTag = SEC_OID_PKCS1_RSA_PSS_SIGNATURE;
+            break;
         case rsaKey:
             switch (hashAlgTag) {
                 case SEC_OID_MD2:
@@ -746,24 +749,45 @@ SEC_GetSignatureAlgorithmOidTag(KeyType keyType, SECOidTag hashAlgTag)
     return sigTag;
 }
 
-static SECItem *
+SECOidTag
+SEC_GetSignatureAlgorithmOidTagByKey(const SECKEYPrivateKey *privKey, const SECKEYPublicKey *pubKey, SECOidTag hashAlgTag)
+{
+    KeyType keyType = nullKey;
+    /* make sure we have a key */
+    if ((privKey && pubKey) || (!privKey && !pubKey)) {
+        PORT_SetError(SEC_ERROR_INVALID_ARGS);
+        return SEC_OID_UNKNOWN;
+    }
+    /* make sure we have only one key */
+    if (privKey) {
+        if (pubKey) {
+            PORT_SetError(SEC_ERROR_INVALID_ARGS);
+            return SEC_OID_UNKNOWN;
+        }
+        keyType = privKey->keyType;
+    } else {
+        /* the logic above should guarentee the following assert. */
+        PORT_Assert(pubKey != NULL);
+        PORT_Assert(privKey == NULL);
+        keyType = pubKey->keyType;
+    }
+    /* for future, we can look at other part of the key to determine the algorithm */
+    return SEC_GetSignatureAlgorithmOidTag(keyType, hashAlgTag);
+}
+
+SECItem *
 sec_CreateRSAPSSParameters(PLArenaPool *arena,
                            SECItem *result,
                            SECOidTag hashAlgTag,
                            const SECItem *params,
-                           const SECKEYPrivateKey *key)
+                           int modBytes)
 {
     SECKEYRSAPSSParams pssParams;
-    int modBytes, hashLength;
+    int hashLength;
     unsigned long saltLength;
     PRBool defaultSHA1 = PR_FALSE;
     SECStatus rv;
 
-    if (key->keyType != rsaKey && key->keyType != rsaPssKey) {
-        PORT_SetError(SEC_ERROR_INVALID_ALGORITHM);
-        return NULL;
-    }
-
     PORT_Memset(&pssParams, 0, sizeof(pssParams));
 
     if (params && params->data) {
@@ -795,10 +819,22 @@ sec_CreateRSAPSSParameters(PLArenaPool *arena,
         }
     }
 
-    modBytes = PK11_GetPrivateModulusLen((SECKEYPrivateKey *)key);
-
     /* Determine the hash algorithm to use, based on hashAlgTag and
-     * pssParams.hashAlg; there are four cases */
+     * pssParams.hashAlg; there are 6  cases.
+     *  case:
+     *  1) We have params and params.hashAlg and we have a specified hashAlgTag,
+     *  make sure that hashAlgTag specified by the appication matches.
+     *  2) We have params, but no params.hashAlg and we have a specified
+     *  hashAlg, make sure the hashAlgTag matches SEC_OID_SHA1.
+     *  3) we did not specify any parameters but we did specified
+     *  a hashAlgTag. Use the specified hash algtag.
+     *  4) We have params and params.hashAlg and we did not specify a
+     *  hashAlgTag, use the hashAlg from the parameter.
+     *  5) We have params, but no params.hashAlg and we did not specify a
+     *  hashAlgTag, use the SEC_OID_SHA1
+     *  6) We did not specify any parameters, nor did we specify a
+     *  hashAlgTag, use the key size to select an appropriate hashAlg.
+     */
     if (hashAlgTag != SEC_OID_UNKNOWN) {
         SECOidTag tag = SEC_OID_UNKNOWN;
 
@@ -829,6 +865,7 @@ sec_CreateRSAPSSParameters(PLArenaPool *arena,
         }
     }
 
+    /* explicitly restrict hashAlg to SHA2 variants */
     if (hashAlgTag != SEC_OID_SHA1 && hashAlgTag != SEC_OID_SHA224 &&
         hashAlgTag != SEC_OID_SHA256 && hashAlgTag != SEC_OID_SHA384 &&
         hashAlgTag != SEC_OID_SHA512) {
@@ -962,6 +999,47 @@ sec_CreateRSAPSSParameters(PLArenaPool *arena,
                               &pssParams, SECKEY_RSAPSSParamsTemplate);
 }
 
+SECItem *
+SEC_CreateRSAPSSParameters(PLArenaPool *arena,
+                           SECItem *result,
+                           SECOidTag hashAlgTag,
+                           const SECItem *params,
+                           const SECKEYPrivateKey *privKey,
+                           const SECKEYPublicKey *pubKey)
+{
+    /* if no keys are provided arrange for the saltLength to be hashLen */
+    int modBytes = (HASH_LENGTH_MAX * 2) + 2;
+
+    /* we don't need all these parameters, but we should have at least
+     * one of these */
+    if (!privKey && !pubKey && !params && (hashAlgTag == SEC_OID_UNKNOWN)) {
+        PORT_SetError(SEC_ERROR_INVALID_ARGS);
+        return NULL;
+    }
+    /* only allow one key */
+    if (privKey && pubKey) {
+        PORT_SetError(SEC_ERROR_INVALID_ARGS);
+        return NULL;
+    }
+    if (privKey) {
+        if (privKey->keyType != rsaKey && privKey->keyType != rsaPssKey) {
+            PORT_SetError(SEC_ERROR_INVALID_ALGORITHM);
+            return NULL;
+        }
+        /* cast away the const, even though it's still logically a const
+         * function */
+        modBytes = PK11_GetPrivateModulusLen((SECKEYPrivateKey *)privKey);
+    } else if (pubKey) {
+        if (pubKey->keyType != rsaKey && pubKey->keyType != rsaPssKey) {
+            PORT_SetError(SEC_ERROR_INVALID_ALGORITHM);
+            return NULL;
+        }
+        modBytes = pubKey->u.rsa.modulus.len;
+    }
+    return sec_CreateRSAPSSParameters(arena, result, hashAlgTag,
+                                      params, modBytes);
+}
+
 SECItem *
 SEC_CreateSignatureAlgorithmParameters(PLArenaPool *arena,
                                        SECItem *result,
@@ -970,18 +1048,102 @@ SEC_CreateSignatureAlgorithmParameters(PLArenaPool *arena,
                                        const SECItem *params,
                                        const SECKEYPrivateKey *key)
 {
+    PORT_SetError(0);
     switch (signAlgTag) {
         case SEC_OID_PKCS1_RSA_PSS_SIGNATURE:
-            return sec_CreateRSAPSSParameters(arena, result,
-                                              hashAlgTag, params, key);
+            return SEC_CreateRSAPSSParameters(arena, result,
+                                              hashAlgTag, params, key, NULL);
 
         default:
             if (params == NULL)
                 return NULL;
             if (result == NULL)
                 result = SECITEM_AllocItem(arena, NULL, 0);
+            if (result == NULL) {
+                return NULL;
+            }
             if (SECITEM_CopyItem(arena, result, params) != SECSuccess)
                 return NULL;
             return result;
     }
 }
+
+SECItem *
+SEC_CreateVerifyAlgorithmParameters(PLArenaPool *arena,
+                                    SECItem *result,
+                                    SECOidTag signAlgTag,
+                                    SECOidTag hashAlgTag,
+                                    const SECItem *params,
+                                    const SECKEYPublicKey *key)
+{
+    PORT_SetError(0);
+    switch (signAlgTag) {
+        case SEC_OID_PKCS1_RSA_PSS_SIGNATURE:
+            return SEC_CreateRSAPSSParameters(arena, result,
+                                              hashAlgTag, params, NULL, key);
+
+        default:
+            if (params == NULL)
+                return NULL;
+            if (result == NULL)
+                result = SECITEM_AllocItem(arena, NULL, 0);
+            if (result == NULL) {
+                return NULL;
+            }
+            if (SECITEM_CopyItem(arena, result, params) != SECSuccess)
+                return NULL;
+            return result;
+    }
+}
+
+SECStatus
+SEC_CreateSignatureAlgorithmID(PLArenaPool *arena,
+                               SECAlgorithmID *signAlgID,
+                               SECOidTag signAlgTag,
+                               SECOidTag hashAlgTag,
+                               const SECItem *params,
+                               const SECKEYPrivateKey *privKey,
+                               const SECKEYPublicKey *pubKey)
+{
+    SECItem *newParams = NULL;
+
+    if (signAlgTag == SEC_OID_UNKNOWN) {
+        signAlgTag = SEC_GetSignatureAlgorithmOidTagByKey(privKey, pubKey,
+                                                          hashAlgTag);
+    } else {
+        /* SEC_GetSignatureAlgorithm already checks if privKey and pubKey
+         * is present and the only case */
+        if ((privKey && pubKey) || (!privKey && !pubKey)) {
+            PORT_SetError(SEC_ERROR_INVALID_ARGS);
+            return SECFailure;
+        }
+    }
+
+    if (signAlgTag == SEC_OID_UNKNOWN) {
+        /* error already set by SEC_GetSignatureAlgorithmOidTagByKey */
+        return SECFailure;
+    }
+
+    if (privKey) {
+        newParams = SEC_CreateSignatureAlgorithmParameters(arena, NULL,
+                                                           signAlgTag,
+                                                           hashAlgTag,
+                                                           params,
+                                                           privKey);
+    } else {
+        /* must be pubKey */
+        newParams = SEC_CreateVerifyAlgorithmParameters(arena, NULL,
+                                                        signAlgTag,
+                                                        hashAlgTag,
+                                                        params,
+                                                        pubKey);
+    }
+
+    /* It's legal (and common) for params to be NULL; look at the error
+     * code to see if there was a failure */
+    if (!newParams && PORT_GetError() != 0) {
+        return SECFailure;
+    }
+
+    return SECOID_SetAlgorithmID(arena, signAlgID, signAlgTag, newParams);
+}
