@@ -449,6 +449,16 @@ static const struct FREEBLVectorStr vector = {
 
     EC_DerivePublicKey,
     /* End of version 3.030 */
+
+    MLDSA_NewKey,
+    MLDSA_SignInit,
+    MLDSA_SignUpdate,
+    MLDSA_SignFinal,
+    MLDSA_VerifyInit,
+    MLDSA_VerifyUpdate,
+    MLDSA_VerifyFinal,
+    /* End of version 3.031 */
+
 };
 
 const FREEBLVector*
