@@ -22,8 +22,8 @@
  * SOFTWARE.
  */
 
-#ifndef __Hacl_Chacha20Poly1305_32_H
-#define __Hacl_Chacha20Poly1305_32_H
+#ifndef __Hacl_Chacha20Poly1305_128_H
+#define __Hacl_Chacha20Poly1305_128_H
 
 #if defined(__cplusplus)
 extern "C" {
@@ -34,8 +34,8 @@ extern "C" {
 #include "krml/lowstar_endianness.h"
 #include "krml/internal/target.h"
 
-#include "Hacl_Poly1305_32.h"
-#include "Hacl_Chacha20.h"
+#include "Hacl_Poly1305_128.h"
+#include "Hacl_Chacha20_Vec128.h"
 
 /**
 Encrypt a message `m` with key `k`.
@@ -54,7 +54,7 @@ Note: Encryption and decryption can be executed in-place, i.e., `m` and `cipher`
 @param mac Pointer to 16 bytes of memory where the mac is written to.
 */
 void
-Hacl_Chacha20Poly1305_32_aead_encrypt(
+Hacl_Chacha20Poly1305_128_aead_encrypt(
     uint8_t *k,
     uint8_t *n,
     uint32_t aadlen,
@@ -86,7 +86,7 @@ If decryption fails, the array `m` remains unchanged and the function returns th
 @returns 0 on succeess; 1 on failure.
 */
 uint32_t
-Hacl_Chacha20Poly1305_32_aead_decrypt(
+Hacl_Chacha20Poly1305_128_aead_decrypt(
     uint8_t *k,
     uint8_t *n,
     uint32_t aadlen,
@@ -100,5 +100,5 @@ Hacl_Chacha20Poly1305_32_aead_decrypt(
 }
 #endif
 
-#define __Hacl_Chacha20Poly1305_32_H_DEFINED
+#define __Hacl_Chacha20Poly1305_128_H_DEFINED
 #endif
