@@ -22,8 +22,8 @@
  * SOFTWARE.
  */
 
-#ifndef __Hacl_Poly1305_32_H
-#define __Hacl_Poly1305_32_H
+#ifndef __Hacl_Poly1305_128_H
+#define __Hacl_Poly1305_128_H
 
 #if defined(__cplusplus)
 extern "C" {
@@ -34,23 +34,31 @@ extern "C" {
 #include "krml/lowstar_endianness.h"
 #include "krml/internal/target.h"
 
-#include "Hacl_Krmllib.h"
+#include "libintvector.h"
 
-typedef uint64_t *Hacl_Poly1305_32_poly1305_ctx;
+typedef Lib_IntVector_Intrinsics_vec128 *Hacl_Poly1305_128_poly1305_ctx;
 
-void Hacl_Poly1305_32_poly1305_init(uint64_t *ctx, uint8_t *key);
+void Hacl_Poly1305_128_poly1305_init(Lib_IntVector_Intrinsics_vec128 *ctx, uint8_t *key);
 
-void Hacl_Poly1305_32_poly1305_update1(uint64_t *ctx, uint8_t *text);
+void Hacl_Poly1305_128_poly1305_update1(Lib_IntVector_Intrinsics_vec128 *ctx, uint8_t *text);
 
-void Hacl_Poly1305_32_poly1305_update(uint64_t *ctx, uint32_t len, uint8_t *text);
+void
+Hacl_Poly1305_128_poly1305_update(
+    Lib_IntVector_Intrinsics_vec128 *ctx,
+    uint32_t len,
+    uint8_t *text);
 
-void Hacl_Poly1305_32_poly1305_finish(uint8_t *tag, uint8_t *key, uint64_t *ctx);
+void
+Hacl_Poly1305_128_poly1305_finish(
+    uint8_t *tag,
+    uint8_t *key,
+    Lib_IntVector_Intrinsics_vec128 *ctx);
 
-void Hacl_Poly1305_32_poly1305_mac(uint8_t *tag, uint32_t len, uint8_t *text, uint8_t *key);
+void Hacl_Poly1305_128_poly1305_mac(uint8_t *tag, uint32_t len, uint8_t *text, uint8_t *key);
 
 #if defined(__cplusplus)
 }
 #endif
 
-#define __Hacl_Poly1305_32_H_DEFINED
+#define __Hacl_Poly1305_128_H_DEFINED
 #endif
