@@ -255,7 +255,7 @@ PK11_UnconfigurePKCS11(void)
  */
 
 static const char *dllname =
-#if defined(XP_WIN32) || defined(XP_OS2)
+#if defined(XP_WIN32)
     "nssckbi.dll";
 #elif defined(HPUX) && !defined(__ia64) /* HP-UX PA-RISC */
     "libnssckbi.sl";
@@ -665,7 +665,7 @@ nss_Init(const char *configdir, const char *certPrefix, const char *keyPrefix,
 
         /* If we're NSS_ContextInit, we're probably a library. It could be
          * possible that the application initialized NSS then forked(). The
-         * library would have no knowledge of that. If we call 
+         * library would have no knowledge of that. If we call
          * SECMOD_RestartModules() here, we will be able to continue on with
          * NSS as normal. SECMOD_RestartModules() does have the side affect
          * of losing all our PKCS #11 objects in the new process, but only if
@@ -746,7 +746,7 @@ nss_Init(const char *configdir, const char *certPrefix, const char *keyPrefix,
                     dbpath += 7;
                 } else if (strncmp(dbpath, "rdb:", 4) == 0) {
                     /* if rdb: is specified, the configdir isn't really a
-               * path. Skip it */
+                     * path. Skip it */
                     dbpath = NULL;
                 }
                 if (dbpath) {
@@ -1323,25 +1323,25 @@ NSS_VersionCheck(const char *importedVersion)
 #define NSS_VERSION_VARIABLE __nss_base_version
 #include "verref.h"
 
-    while (isdigit(*ptr)) {
+    while (isdigit((unsigned char)*ptr)) {
         vmajor = 10 * vmajor + *ptr - '0';
         ptr++;
     }
     if (*ptr == '.') {
         ptr++;
-        while (isdigit(*ptr)) {
+        while (isdigit((unsigned char)*ptr)) {
             vminor = 10 * vminor + *ptr - '0';
             ptr++;
         }
         if (*ptr == '.') {
             ptr++;
-            while (isdigit(*ptr)) {
+            while (isdigit((unsigned char)*ptr)) {
                 vpatch = 10 * vpatch + *ptr - '0';
                 ptr++;
             }
             if (*ptr == '.') {
                 ptr++;
-                while (isdigit(*ptr)) {
+                while (isdigit((unsigned char)*ptr)) {
                     vbuild = 10 * vbuild + *ptr - '0';
                     ptr++;
                 }
