@@ -77,7 +77,8 @@ PRBool lg_FIPSEntryOK(void);
 #define LG_TOKEN_TYPE_PUB 0x10000000L
 #define LG_TOKEN_TYPE_KEY 0x18000000L
 /* certdb (high bit == 1) */
-#define LG_TOKEN_TYPE_TRUST 0x20000000L
+#define LG_TOKEN_TYPE_TRUST 0x00000000L
+#define LG_TOKEN_TYPE_NSS_TRUST 0x20000000L
 #define LG_TOKEN_TYPE_CRL 0x28000000L
 #define LG_TOKEN_TYPE_SMIME 0x30000000L
 #define LG_TOKEN_TYPE_CERT 0x38000000L
@@ -150,6 +151,8 @@ CK_RV lg_Abort(SDB *sdb);
 CK_RV lg_GetMetaData(SDB *sdb, const char *id, SECItem *item1, SECItem *item2);
 CK_RV lg_PutMetaData(SDB *sdb, const char *id,
                      const SECItem *item1, const SECItem *item2);
+CK_RV lg_DestroyMetaData(SDB *sdb, const char *id);
+CK_RV lg_GetNewObjectID(SDB *sdb, CK_OBJECT_HANDLE *object_id);
 
 SEC_END_PROTOS
 
