@@ -274,7 +274,7 @@ NSSUTIL_ArgDecodeNumber(const char *num)
     }
 
     for (; *num; num++) {
-        if (isdigit(*num)) {
+        if (isdigit((unsigned char)*num)) {
             digit = *num - '0';
         } else if ((*num >= 'a') && (*num <= 'f')) {
             digit = *num - 'a' + 10;
@@ -585,7 +585,7 @@ struct nssutilArgSlotFlagTable {
 
 #define NSSUTIL_ARG_ENTRY(arg, flag) \
     {                                \
-        #arg, sizeof(#arg) - 1, flag \
+#arg, sizeof(#arg) - 1, flag \
     }
 static struct nssutilArgSlotFlagTable nssutil_argSlotFlagTable[] = {
     NSSUTIL_ARG_ENTRY(RSA, SECMOD_RSA_FLAG),
